//Se incluye el archivo de configuración donde se encuentran las definiciones de pines, importación de librerías y declaración de variables necesarias para la ejecución del programa
#include "ConfigENT.h"

//Se crea el string que contendrá el nombre del trabajador vinculado a la UID que se aproxime al sensor RFID
String trabajador;

//Se crea el objeto de la clase Servo que servirá para gestionar el movimiento del mismo
Servo cerradura;

//Se crea el objeto de la clase MFRC que permitirá gestionar el sistema RFID
MFRC522 rfid(SS_PIN, RST_PIN);
MFRC522::MIFARE_Key key;
byte nuidPICC[4];

//Se enlazan los valores Hexadecimales de la tarjeta y el llavero con el nombre de dos empleados
String CodHex;

// Se conecta al feed tfg, que es donde se encuentra el LED AZUL que servirá de trigger para el movimiento del servo al usar el comando de voz
AdafruitIO_Feed *digital = io.feed("tfg");

//------------------------------------------------------------------------------------------------------------------------------------------------------
//El setup es la parte del código que se ejecuta una única vez

void setup() {
  
//Se inicializa la comunicación serie entre el microcontrolador y el PC y la comunicación SPI entre el sensor RFID y el microcontrolador
    Serial.begin(115200);
    SPI.begin();

//Se establece que el PIN de datos al que está conectado el Servo es el declarado en el fichero de configuración independiente
     cerradura.attach(servoPin);

//Se coloca el servomotor en posición 0º (cerrado)      
      cerradura.write(0);
      
//Se inicializa el chip del módulo MFRC52
      rfid.PCD_Init();

 //Se recorren todos los sectores de la memoria del identificador aproximado para mostrar en pantalla su UID en hexadecimal y así compararlo con los declarados como los asignados a los nombres de 2 empleados
      for(byte i=0;i<6;i++){
        key.keyByte[i]= 0xFF;
        }//for

//Se muestra en pantalla el UID del identificador en hexadecimal

        CodHex=printHex(key.keyByte, MFRC522::MF_KEY_SIZE);

        
//Se establece que los pines a los que están conectados los diferentes diodos LED, se comporten como pines de salida

  pinMode(LED_ROJO, OUTPUT); //LED ROJO
  pinMode(LED_VERDE, OUTPUT); //LED VERDE
  pinMode(LED_AZUL, OUTPUT); //LED AZUL

//Se realiza la conexión con el servidor de Adafruit.io con los parámetros configurados anteriormente
  io.connect();

// Se crea una función que almacene los datos (1 o 0) recibidos desde el "FEED" de Adafruit.IO y los use para activar el movimiento del servomotor en función de los mismos
  digital->onMessage(instruccion);

 // Mientras la conexión no se haya realizado muestra puntos suspensivos en el monitor serie a modo de spinner de carga
  while(io.status() < AIO_CONNECTED) {
    Serial.print(".");
    delay(500);
  }

  // Una vez sale del bucle while, lo cual indica que ya se ha conectado satisfactoriamente, muestra el statusText que indicará que se ha conectado a Adafruit.io
  Serial.println();
  Serial.println(io.statusText());
  digital->get();  




}//setup

//--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

 //El loop es la parte del código que se ejecuta en bucle constantemente, mientras la placa tenga alimentación
 
void loop() {


  //La función run mantiene constantemente la conexión del ESP32 con el servidor de ADAFRUIT.io
   io.run();

    

  //Se crean dos bucles para comprobar si se han aproximado tarjetas de RFID , y en ese caso se almacena su tipo para después comprobar si es el correcto
  
    if ( ! rfid.PICC_IsNewCardPresent()){return;}
    if ( ! rfid.PICC_ReadCardSerial()){return;}

    Serial.println(F("PICC type: "));
    MFRC522::PICC_Type piccType= rfid.PICC_GetType(rfid.uid.sak);
    Serial.println(rfid.PICC_GetTypeName(piccType));

  
     // Se comprueba si el tipo de tarjeta/llavero RFID es del tipo MIFARE, que es el que lee este lector

     
     if (piccType != MFRC522::PICC_TYPE_MIFARE_MINI && piccType != MFRC522::PICC_TYPE_MIFARE_1K && piccType != MFRC522::PICC_TYPE_MIFARE_4K)
     {
       Serial.println("El identificador aproximado no es MIFARE Classic, por lo que no funciona en este dispositivo");
       return;
     }
     
     if (rfid.uid.uidByte[0] != nuidPICC[0] || rfid.uid.uidByte[1] != nuidPICC[1] || rfid.uid.uidByte[2] != nuidPICC[2] || rfid.uid.uidByte[3] != nuidPICC[3] )
     {

    // Se muestra en el monitor serie un mensaje de que se ha detectado una tarjeta del tipo correcto 
      
       Serial.println("Se ha detectado una nueva tarjeta de identificacion");
       
       // Se almacenan los datos hexadecimales de la tarjeta en la variable CodHex para poder comprobar si se trata de una ID registrada
       
       for (byte i = 0; i < 4; i++) {nuidPICC[i] = rfid.uid.uidByte[i];}
    
       CodHex = printHex(rfid.uid.uidByte, rfid.uid.size);


 // Se realizan las comprobaciones con las tarjetas registradas en el sistema, si es correcta, se encenderá el LED VERDE, se abrirá la cerradura ( movimiento del servo a 180º), se muestra un mensaje en el monitor serie, se espera 4 segundos para dar tiempo a que cierre la puerta, y se cierra la cerradura y se apaga el LED
       if(ANDRES == CodHex)
       {
        Serial.println("Bienvenido ANDRES");
        delay(500);
        digitalWrite(LED_VERDE,HIGH);
        delay(500);
        cerradura.write(180);
        delay(500);
        digitalWrite(LED_VERDE,LOW);
        Serial.println(" SE HA ABIERTO LA PUERTA");
        delay(4000);
        
//Se establece que el trabajador que ha accedido es "Andres" puesto que el UID de su identificador en Hexadecimal coincide con el aproximado al sensor, por lo que se almacena su nombre y apellidos junto a una contrabarra en el string "trabajador"

        trabajador="\"Andres.Madariaga\"";

//Se llama a la función insertarenDB que realiza la inserción en base de datos del acceso que se acaba de realizar        
        insertarenDB(trabajador);
        cerradura.write(0);
        delay(200);

  //Cada vez que se realiza la operación de identificación de una tarjeta, se resetea la placa puesto que el sensor tiene memoria y no detecta como nueva una tarjeta ya aproximada anteriormente 
       
        ESP.restart();
       }//ifAndres
       
//Si el UID del identificador aproximado coincide con el de "Ana" se realizan las mismas funciones que en el caso anterior, cambiando el nombre del trabajador, pudiéndose escalar esto a los trabajadores que se quiera     
       else if(ANA == CodHex)
       {
        Serial.println("Bienvenida ANA");
        delay(500);
        digitalWrite(LED_VERDE,HIGH);
        delay(500);
        cerradura.write(180);
        delay(500);
        digitalWrite(LED_VERDE,LOW);
        Serial.println(" SE HA ABIERTO LA PUERTA");
        delay(4000);
        trabajador="\"Ana.Perez\"";
        insertarenDB(trabajador);
        cerradura.write(0);
        delay(2000);
        ESP.restart();
       }//ifAna
       else

//Si la ID aproximada no corresponde con una de las registradas, se muestra un mensaje que indica que es una tarjeta desconocida y enciende el LED ROJO para dar una indicación más visual al usuario
      
       {
        digitalWrite(LED_ROJO,HIGH);
        Serial.println("TARJETA DE IDENTIFICACION DESCONOCIDA");
        delay(600);
        digitalWrite(LED_ROJO,LOW);

//Se realiza un reseteo de la placa por lo mencionado anteriormente
        
        ESP.restart();
        
        
       }//else  

     }//if RFID

   

     // Se detiene la lectura del sensor RFID
     rfid.PICC_HaltA();
     
     // Se detiene el cifrado en PCD
     
     rfid.PCD_StopCrypto1();
    
            
}//loop


// Se crea la funcion que muestra en el monitor el código Hexadecimal de la tarjeta que se ha aproximado al sensor RFID

String printHex(byte *buffer, byte bufferSize)
{  
   String DatoHexAux = "";
   for (byte i = 0; i < bufferSize; i++  ) 
   {
       if (buffer[i] < 0x10)
       {
        DatoHexAux = DatoHexAux + "0";
        DatoHexAux = DatoHexAux + String(buffer[i], HEX);  
       }
       else { DatoHexAux = DatoHexAux + String(buffer[i], HEX); }
   }
   
   for (int i = 0; i < DatoHexAux.length();i++) {DatoHexAux[i] = toupper(DatoHexAux[i]);}
   return DatoHexAux;
}//printHex 




//Se crea una funcion instruccion que leerá el dato que se recibe desde ADAFRUIT.IO y actuará en consecuencia en los dispositivos del microcontrolador ESP32  

void instruccion(AdafruitIO_Data *data) {

//Como se ha comentado con anterioridad, el LED AZUL se usa como trigger para mover la cerradura, y también indicará al usuario que la cerradura ha sido accionada mediante la skill implementada en Google Assisstant

  //Si se recibe un dato pasado a nivel de Pin Digital "HIGH", querrá decir que se ha recibido un 1, y esto será asi porque se ha configurado el envío de un "1" a Adafruit.IO al ussar la instrucción de APERTURA de puerta
  if(data->toPinLevel() == HIGH){

  //Por lo tanto, se muestra que se ha indicado "ABRIR" en el monitor serie
  
    Serial.println("Alguien ha indicado al asistente de Google que la cerradura se debe ABRIR");
    
  //Se enciende el LED AZUL, ya que el servomotor se moverá por la instrucción recibida a través del asistente de google
    digitalWrite(LED_AZUL, data->toPinLevel());
     delay(500);
     
  //Se realiza el movimiento del servomotor, y tras 4 segundos, se mueve de nuevo para "cerrarlo", y se apaga el LED Azul indicando que la operación ha terminado
     cerradura.write(180);
     delay(4000);
     cerradura.write(0);
     delay(500);
     digitalWrite(LED_AZUL,LOW);
  }//if

  
  //Si recibe un "0", quiere decir que la puerta se encuentra cerrada
  else{
    Serial.println("La puerta se encuentra cerrada");
    
    }//else

}//instruccion


//Esta funcion recibe como parámetro el String con el nombre del trabajador y lo inserta en la base de datos alojada en un servidor web  
  void insertarenDB(String trabajador)
 {

//Se crea un string que se añadira a la url del nombre del servidor, el nombre del trabajador, y que será utilizado en la petición HTTP que llamará al script PHP alojado en el servidor WEB
  String query= "?trabajador="+String(trabajador);

//Se crea un objeto de la clase HTTPClient para poder hacer la petición HTTP  
  HTTPClient http;

//Se realiza la petición HTTP, concatenando el nombre del servidor web donde se aloja el script PHP y el nombre del trabajador para que se ejecute la query de inserción del nombre en la base de datos
  http.begin(SERVERNAME+ query); //HTTP

//Se almacena el código que devuelva la petición HTTP en httpCode  
  int httpCode = http.GET();

  
// Si se produce un error en la petición, el código será negativo
  if(httpCode > 0) {

//Una vez se conoce que el código es positivo, querrá decir que el fichero ha sido localizado en el servidor, se comprueba que se devuelve un HTTP_CODE_OK que querrá decir que todo se ha ejecutado satisfactoriamente
    if(httpCode == HTTP_CODE_OK) {
      
//Se almacena el payload que devuelve la petición y se muestra en pantalla, esta respuesta se ha configurado en el script PHP

      String payload = http.getString();
      Serial.println(payload);
    } else {
      
//La cabecera de la petición HTTP se ha enviado y la respuesta recibida ha sido almacenada, se muestra en pantalla

      Serial.printf("[HTTP] GET... code: %d\n", httpCode);
    }
  } else {

//Si surge algún otro error durante la petición se pasa a string y se muestra en el monitor serie
    
    Serial.printf("[HTTP] GET... failed, error: %s\n", http.errorToString(httpCode).c_str());
  }

//Se finaliza la petición HTTP

  http.end();  
  }//insertarenDB
