#include "ConfigINT.h"

//Se inicializa el sensor DHT11 que medirá la temperatura y humedad y el WiFiClient que se usará para enviar los datos a ThingSpeak
WiFiClient cliente;
DHT dht1(pinDHT,DHT11);

// Se selecciona el feed de los que he creado en Adafruit.io para saber dónde se enviarán los datos
AdafruitIO_Feed *digital = io.feed("tfg2");

//Se declaran los dos posibles estados del boton para que encienda/apague el ventilador
int estadoBoton; //estado actual del boton
int ultimo_estadoBoton;//estado previo del boton
int estadoRele = HIGH ; //estado del Relé(sin funcionar)



//--------------------------------------------------------------------------------------------------------------------------------------

//El setup es la parte del código que se ejecuta una única vez

void setup() {

  //Se inicializa el puerto serie de la placa y la conexión a la red WiFi utilizando las credenciales almacenadas en las variables que se crearon anteriormente
  Serial.begin(115200);
  WiFi.begin(WIFI_SSID,WIFI_PASS);

  //Se declara un bucle que, mientras no se haya realizado la conexión WiFi, muestre un mensaje de carga al usuario
  while (WiFi.status() != WL_CONNECTED) {
      delay(100);
      Serial.println("Conectando a la red WiFi...");
    
  }

  // Al salir del bucle while se muestra un mensaje de que la conexión a WiFi se ha realizado de forma satisfactoria
  Serial.println("La red WiFi está conectada");

  //Se inicializa la conexión a ThingSpeak mediante el WiFiClient antes declarado y la lectura del sensor DHT11
  ThingSpeak.begin(cliente);
  dht1.begin();

  //Se declaran como OUTPUT los pines a los que están conectados tanto el LED Rojo como el RELÉ, además del botón como Pullup interno
  
  pinMode(LEDROJO, OUTPUT);
  pinMode(PINRELE, OUTPUT);
  pinMode(BOTON,INPUT_PULLUP);

//Se lee el estado actual del boton (HIGH o LOW)
  estadoBoton = digitalRead (BOTON);

  

 //Se realiza la conexión con el servidor de Adafruit.io con los parámetros configurados anteriormente
  io.connect();
  
// Se crea una función que almacene los datos (1 o 0) recibidos desde el "FEED" de Adafruit.IO y los use para activar el movimiento del servomotor en función de los mismos

  digital->onMessage(instruccion);

  // Mientras la conexión no se haya realizado muestra puntos suspensivos por el monitor serie a modo de mensaje de carga
  while(io.status() < AIO_CONNECTED) {
    Serial.print(".");
    delay(500);
  }

  // Una vez sale del bucle while, lo cual indica que ya se ha conectado satisfactoriamente, muestra el statusText que indicará que se ha conectado a Adafruit.io
  Serial.println();
  Serial.println(io.statusText());
  digital->get();
  
}//setup

//------------------------------------------------------------------------------------------------------------------------------------------------------------

 //El loop es la parte del código que se ejecuta en bucle constantemente, mientras la placa tenga alimentación
 
void loop() {

//La función run mantiene constantemente la conexión del ESP32 con el servidor de ADAFRUIT.io
  io.run();


//En primer lugar se asigna el estado previo del boton al estado actual, para comprobar si cambia
ultimo_estadoBoton = estadoBoton;

//Se lee el estado del pin digital al que está conectado el boton 
estadoBoton = digitalRead (BOTON);

//Si el estado del botón ha cambiado, es decir, se ha pulsado el botón, se muestra en pantalla un mensaje que así lo indica
  if(ultimo_estadoBoton == HIGH && estadoBoton == LOW){
  
  
      Serial.println ("Se ha pulsado el interruptor");

//Se invierte el estado almacenado en estadoRele, para encenderlo o apagarlo
      estadoRele = !estadoRele;

//Se cambia el estado del PIN al que está conectado el relé, indicándole el estado que deberá tener
 
      digitalWrite(PINRELE, estadoRele);
  
  }//if

  
//Se establece un delay de 2000ms puesto que ese es el ritmo de lectura del sensor DHT11 
delay(2000);

//Se llama a la función que leerá la temperatura y la humedad del interior de la sala
leerdht1();

//Se envían a ThingSpeak, usando las credenciales del servidor declaradas en variables anteriormente, las lecturas del sensor DHT11

ThingSpeak.writeFields(channelID, WriteAPIKey);
Serial.println("Se han enviado los registros a ThingSpeak");
Serial.println("---------------------------------------------");

//Se establece un delay de 15000ms puesto que la versión gratuita de ThingSpeak establece un retardo entre escrituras de datos de 14000ms
delay(15000);




}//loop

//Para tener un código más limpio y eficiente, la función que capta los datos mediante el DHT11 y los envía a ThingSpeak se hace fuera del loop

void leerdht1(){
  
   //Se llama a las funciones del DHT11 que leen la temperatura y la humedad
   
  float t1=dht1.readTemperature();
  float h1=dht1.readHumidity();

 //Si hay algún error de lectura, se muestra un mensaje al usuario y se prueba a volver a realizarla, solo entra al bucle si no recibe números en las lecturas
 
  while (isnan(t1)||isnan(h1)){
    
    Serial.println("Ha habido un error en la lectura");
    delay(2000);
    t1=dht1.readTemperature();
    h1=dht1.readHumidity();
    
     }//while

//Se muestran en el Monitor Serie los datos obtenidos

   Serial.print("Temperatura: ");
   Serial.print(t1);
   Serial.println(" ºC. ");

   Serial.print("Humedad: ");
   Serial.print(h1);
   Serial.println(" %. ");


   Serial.println("-------------------------");

//Si la temperatura es superior a 25º, se enciende un LED rojo que indicará que hay una alta temperatura
  
   if(t1>25){
 
     digitalWrite(LEDROJO,HIGH);  
       
    }//if t1



//Se hace la relación entre los datos captados y el número de identificación con el que serán tratados en ThingSpeak

   ThingSpeak.setField(1,t1);
   ThingSpeak.setField(2,h1);

  }//leerdht1



//Se crea la función que tratará el valor que se recibe desde el servidor de ADAFRUIT.io, que será un 1 o un 0

  void instruccion (AdafruitIO_Data *data) {

//Se muestra un mensaje que indique al usuario el dato que se ha recibido

  Serial.print("Alguien ha usado un comando para <- ");

//Si el dato recibido es "HIGH" (1) mostrará que se ha APAGADO la ventilación y mediante la función digitalWrite aplicará ese valor al pin al que está conectado el RELÉ
 
  if(data->toPinLevel() == HIGH){

    
    Serial.println("APAGAR LA VENTILACION");
    digitalWrite(PINRELE, data->toPinLevel());


     
  }//if

//Si no, se habrá recibido un LOW (0) por lo que se habrá ENCENDIDO la ventilación y se usa la función digitalWrite para aplicar ese valor al pin al que está conectado el RELÉ, que se activará al ser un RELÉ de activación a NIVEL BAJO
  else{
    Serial.println("ENCENDER LA VENTILACION");
    digitalWrite(PINRELE,data->toPinLevel());
    
    
    }//else

}//instruccion
